# Generated by Django 5.0.6 on 2024-07-11 14:52

from django.db import migrations, models
from django.db.models import CharField, F, OuterRef, Subquery, Value
from django.db.models.functions import Concat
from enumfields import Enum

from arkindex.project.tools import RTrimChr


class GitRefType(Enum):
    Branch = "branch"
    Tag = "tag"


def update_git_refs(apps, schema):
    Worker = apps.get_model("process", "Worker")
    WorkerVersion = apps.get_model("process", "WorkerVersion")

    Worker.objects.update(
        repository_url=Subquery(
            Worker.objects
            .filter(pk=OuterRef("pk"))
            .values("repository__url")
        )
    )

    WorkerVersion.objects.update(
        revision_url=Subquery(
            WorkerVersion.objects
            .filter(pk=OuterRef("pk"))
            .exclude(worker__repository_id=None)
            .exclude(revision_id=None)
            .annotate(
                rev_url=Concat(
                    RTrimChr("worker__repository__url", Value("/")),
                    Value("/commit/"),
                    F("revision__hash"),
                    output_field=CharField(),
                )
            )
            .values("rev_url")
        ),
        tag=Subquery(
            WorkerVersion.objects
            .filter(
                pk=OuterRef("pk"),
                revision__refs__type=GitRefType.Tag.value,
            )
            .order_by("-revision__refs__name")
            [:1]
            .values("revision__refs__name")
        )
    )


class Migration(migrations.Migration):

    dependencies = [
        ("documents", "0012_alter_transcriptionentity_id"),
        ("process", "0039_worker_configuration_name_not_empty"),
    ]

    operations = [
        migrations.AddField(
            model_name="worker",
            name="repository_url",
            field=models.URLField(blank=True, default=None, max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="workerversion",
            name="revision_url",
            field=models.URLField(blank=True, default=None, max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="workerversion",
            name="tag",
            field=models.CharField(blank=True, default=None, max_length=50, null=True),
        ),
        migrations.AddConstraint(
            model_name="workerversion",
            constraint=models.UniqueConstraint(condition=models.Q(("tag__isnull", False)), fields=("worker", "tag"), name="workerversion_unique_tag"),
        ),
        migrations.RunPython(
            update_git_refs,
            reverse_code=migrations.RunPython.noop,
            elidable=True,
        ),
    ]
